<?php

use CRM_Eventics_ExtensionUtil as E;

/**
 * Form controller class
 *
 * @see https://docs.civicrm.org/dev/en/latest/framework/quickform/
 */
class CRM_Eventics_Form_Eventics extends CRM_Event_Form_ManageEvent {
  public $_component = 'event';

  public function preProcess() : void {
    parent::preProcess();
    $this->setSelectedChild('eventics');
  }

  public function buildQuickForm() : void {
    $this->add('advcheckbox', 'ics_enabled', E::ts('Attach a calendar invite?'));
    $this->add('text', 'event_location', E::ts('Location'), ['class' => 'huge40']);
    $this->add('textarea', 'event_description', E::ts('Description'), ['class' => 'big']);
    $this->add('select', 'event_timezone', E::ts('Timezone'), ['' => E::ts('- select -')] + CRM_Eventics_Utils_Helper::getTimezones(), FALSE, ['class' => 'crm-select2']);

    // Export form elements
    $this->assign('elementNames', $this->getRenderableElementNames());
    parent::buildQuickForm();
  }

  public function setDefaultValues() : array {
    $defaults = [];
    $defaults['ics_enabled'] = Civi::settings()->get('eventics_default_send_ics');

    $smarty = CRM_Core_Smarty::singleton();
    $smarty->assign('defaultSiteTimezone', CRM_Eventics_Utils_Helper::getSiteDefaultTimezone());

    if (isset($this->_id)) {
      $eventics = \Civi\Api4\Eventics::get(FALSE)
        ->addWhere('event_id', '=', $this->_id)
        ->execute()
        ->first();

      if ($eventics) {
        $defaults = $eventics;
      }
    }

    return $defaults;
  }

  public function postProcess() {
    $values = $this->exportValues();
    $event_id = $this->_id;
    $eventics = \Civi\Api4\Eventics::get(FALSE)
      ->addWhere('event_id', '=', $event_id)
      ->execute()
      ->first();

    if (!empty($eventics)) {
      \Civi\Api4\Eventics::update(FALSE)
        ->addWhere('event_id', '=', $event_id)
        ->addValue('ics_enabled', $values['ics_enabled'])
        ->addValue('event_location', $values['event_location'])
        ->addValue('event_description', $values['event_description'])
        ->addValue('event_timezone', $values['event_timezone'])
        ->execute();
    }
    else {
      \Civi\Api4\Eventics::create(FALSE)
        ->addValue('event_id', $event_id)
        ->addValue('ics_enabled', $values['ics_enabled'])
        ->addValue('event_location', $values['event_location'])
        ->addValue('event_description', $values['event_description'])
        ->addValue('event_timezone', $values['event_timezone'])
        ->execute();
    }

    parent::endPostProcess();
  }

  /**
   * Get the fields/elements defined in this form.
   *
   * @return array (string)
   */
  public function getRenderableElementNames() {
    // The _elements list includes some items which should not be
    // auto-rendered in the loop -- such as "qfKey" and "buttons".  These
    // items don't have labels.  We'll identify renderable by filtering on
    // the 'label'.
    $elementNames = [];
    foreach ($this->_elements as $element) {
      /** @var HTML_QuickForm_Element $element */
      $label = $element->getLabel();
      if (!empty($label)) {
        $elementNames[] = $element->getName();
      }
    }
    return $elementNames;
  }

}
