<?php

require_once 'eventics.civix.php';
// phpcs:disable
use CRM_Eventics_ExtensionUtil as E;
// phpcs:enable

/**
 * Implements hook_civicrm_config().
 *
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_config/
 */
function eventics_civicrm_config(&$config) {
  _eventics_civix_civicrm_config($config);
}

/**
 * Implements hook_civicrm_container().
 *
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_container/
 */
function eventics_civicrm_container($container) {
  $container->addResource(new \Symfony\Component\Config\Resource\FileResource(__FILE__));
  $container->findDefinition('dispatcher')->addMethodCall('addListener', ['civi.token.eval', 'eventics_evaluate_tokens'])->setPublic(true);
  $container->findDefinition('dispatcher')->addMethodCall('addListener', ['civi.token.list', 'eventics_list_tokens'])->setPublic(true);
}

/**
 * Implements hook_civicrm_install().
 *
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_install
 */
function eventics_civicrm_install() {
  _eventics_civix_civicrm_install();
}

/**
 * Implements hook_civicrm_enable().
 *
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_enable
 */
function eventics_civicrm_enable() {
  _eventics_civix_civicrm_enable();
}

/**
 * Implements hook_civicrm_alterMailParams().
 *
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_alterMailParams
 */
function eventics_civicrm_alterMailParams(&$params, $context) {
  // Single email sent from the Participant Search
  // or CiviMail if using a specific token such as eventics.participant_eventics_attachment.1234
  if (!empty(\Civi::$statics['eventics']['alterMail'])) {
    // eventics_evaluate_tokens will have inserted this pseudo-token in the mail body
    // so we remove it, and attach an image instead
    // @todo This does not support tokens for links specific to Outlook or Gmail
    if (preg_match_all('/EVENTICS##ATT#(\d+)##/', $params['html'], $matches)) {
      foreach ($matches[1] as $participant_id) {
        $participant = \Civi\Api4\Participant::get(FALSE)
          ->addSelect('event_id')
          ->addWhere('id', '=', $participant_id)
          ->execute()
          ->first();

        $attachment = CRM_Eventics_Utils_Helper::generateIcsAttachment($participant['event_id'], $participant_id);

        if ($attachment) {
          $params['attachments'][] = $attachment;
        }
      }

      // Cleaup the mail body
      $params['html'] = preg_replace('/EVENTICS##ATT#(\d+)##/', '', $params['html']);
      $params['text'] = preg_replace('/EVENTICS##ATT#(\d+)##/', '', $params['text']);
    }
    return;
  }

  if ($context != 'messageTemplate') {
    return;
  }

  $name = $params['valueName'] ?? NULL;

  if ($name == 'event_online_receipt' || $name == 'event_offline_receipt') {
    // Respectively: online receipt, offline receipt
    $event_id = $params['tplParams']['event']['id'] ?? $params['tplParams']['eventID'] ?? NULL;
    $participant_id = $params['tplParams']['participantID'] ?? NULL;

    if (!$event_id) {
      return;
    }

    $attachment = CRM_Eventics_Utils_Helper::generateIcsAttachment($event_id, $participant_id);

    if (!empty($attachment)) {
      $params['attachments'][] = $attachment;

      $googleLink = Civi::settings()->get('eventics_send_google_link');
      $outlookLink = Civi::settings()->get('eventics_send_outlook_link');

      // Create Outlook LInk
      if ($outlookLink) {
        $url = str_replace(Civi::paths()->getPath($moveTo), \Civi::paths()->getUrl($moveTo, 'absolute'), $icsfile);
        // FIXME: add in README that we need to add this to the template
        $params['tplParams']['icslinks']['icslink'] = $url;
      }

      // Create Google Link
      if ($googleLink) {
        foreach ($info as $event) {
          $url = CRM_Eventics_Utils_Helper::generateGoogleCalendarLink($event['title'], $event['start_date'], $event['end_date'], $event['location'], $event['description'], $event['timezone']);
          $params['tplParams']['icslinks']['gcalendarlink'] = $url;
        }
      }
    }
  }
}

/**
 * Implements hook_civicrm_tabset().
 */
function eventics_civicrm_tabset($tabsetName, &$tabs, $context) {
  if ($tabsetName == 'civicrm/event/manage') {
    if (!empty($context)) {
      $eventID = $context['event_id'];
      $eventicses = \Civi\Api4\Eventics::get(FALSE)
        ->addWhere('event_id', '=', $eventID)
        ->addSelect('ics_enabled')
        ->execute();
      $url = CRM_Utils_System::url('civicrm/event/manage/eventics', "reset=1&id=$eventID&action=update&component=event");
      // Add a new eventics tab along with url
      $tab['eventics'] = [
        'title' => E::ts('Send iCal'),
        'link' => $url,
        'valid' => Civi::settings()->get('eventics_default_send_ics') ?: $eventicses[0]['ics_enabled'] ?? NULL,
        'active' => 1,
        'current' => false,
      ];
    }
    else {
      $tab['eventics'] = [
        'title' => E::ts('Send iCal'),
        'url' => 'civicrm/event/manage/eventics',
        'field' => 'ics_enabled',
      ];
    }
    // Insert this tab into position 4
    $tabs = array_merge(
      array_slice($tabs, 0, 4),
      $tab,
      array_slice($tabs, 4)
    );
  }

  if ($tabsetName == 'civicrm/event/manage/rows') {
    $eventID = $context['event_id'];
    if ($eventID) {
      $eventicses = \Civi\Api4\Eventics::get(FALSE)
        ->addWhere('event_id', '=', $eventID)
        ->addSelect('ics_enabled')
        ->execute();
      $tabs[$eventID]['ics_enabled'] = $eventicses[0]['ics_enabled'] ?? Civi::settings()->get('eventics_default_send_ics') ?: NULL;
    }
  }
}

/**
 * Symfony event for civi.token.eval
 *
 * @param \Civi\Token\Event\TokenValueEvent $event
 */
function eventics_evaluate_tokens(Civi\Token\Event\TokenValueEvent $event) {
  // We set this when we need to attach an ics file and we need to make
  // sure it gets NULLed between two reminders to avoid stale values.
  \Civi::$statics['eventics']['participant_id'] = NULL;

  // Check if our tokens are in the message
  $messageTokens = $event->getTokenProcessor()->getMessageTokens();
  if (empty($messageTokens['eventics'])) {
    return;
  }

  foreach ($event->getRows() as $row) {
    // Check for a generic ics token for any event, but where we know the participant_id
    if (!empty($row->context['participantId'])) {
      if (in_array('participant_ics_attachment', $messageTokens['eventics'])) {
        // This is used by eventics_civicrm_alterMailParams
        $row->tokens('eventics', 'participant_ics_attachment', 'EVENTICS##ATT#' . $row->context['participantId'] . '##');
        \Civi::$statics['eventics']['alterMail'] = true;
      }
    }

    // Inspect all tokens, to see if we are generating an ICS file for a specific event
    foreach ($messageTokens['eventics'] as $token) {
      // Look for tokens that look like 'eventics.participant_ics_attachment.1234'
      // the eventics part is already extracted, so the token looks like participant_ics_attachment.1234
      $parts = explode('.', $token);

      if (count($parts) != 2) {
        continue;
      }

      if ($parts[0] == 'participant_ics_attachment') {
        if (!is_numeric($parts[1])) {
          Civi::log()->warning("eventics: ignoring token $token. The last part does not seem to be numeric.");
          continue;
        }

        $event_id = $parts[1];
        $contact_id = $row->context['contactId'] ?? NULL;

        if (!$contact_id) {
          Civi::log()->warning("eventics: ignoring token $token because I could not find a contactId in the context");
          continue;
        }

        // Check if this contact has a participant record for this event
        $participant = \Civi\Api4\Participant::get(FALSE)
          ->addSelect('id')
          ->addWhere('event_id', '=', $event_id)
          ->addWhere('contact_id', '=', $contact_id)
          ->execute()
          ->first();

        if (!$participant) {
          Civi::log()->warning("eventics: ignoring token $token because CID $contact_id is not a participant for the event $event_id");
          continue;
        }

        $participant_id = $participant['id'];
        $row->format('text/html');
        $row->tokens('eventics', 'participant_ics_attachment.' . $event_id, 'EVENTICS##ATT#' . $participant_id . '##');
        \Civi::$statics['eventics']['alterMail'] = true;
      }
    }
  }
}

/**
 * Symfony event for civi.token.list
 */
function eventics_list_tokens(\Civi\Token\Event\TokenRegisterEvent $event) {
  $event->entity('eventics')
    ->register('participant_ics_attachment', E::ts('Event iCal (ics) Attachment'));

  // Add all active events
  $events = \Civi\Api4\Event::get(TRUE)
    ->addSelect('id', 'title')
    ->addWhere('start_date', '>=', 'NOW()')
    ->addClause('OR', ['end_date', '>=', 'NOW()'])
    ->execute();

  foreach ($events as $ev) {
    $event->entity('eventics')
      ->register('participant_ics_attachment.' . $ev['id'], E::ts('Event iCal (ics) Attachment for: %1', [1 => $ev['title']]));
  }
}
